-- Daily Global Check
-- by Jadya
-- EU-Well of Eternity

local addonName, addonTable = ...
local L = addonTable.L
local C = addonTable.C

local AddColor = addonTable.AddColor

local constructors = DailyGlobalCheck.Constructors
local options      = DailyGlobalCheck.OptFunctions
local maps         = DailyGlobalCheck.MapFunctions
local lists        = DailyGlobalCheck.ListFunctions

local addontitle = "|TInterface\\icons\\INV_Sigil_Freya:16 |t|cff00AAFFDaily Global Check|r"
local dgcicon = "Interface\\icons\\INV_Sigil_Freya"

local frameoffsetY = -60

------ DGC Base Options ------
function options:OnOptionChanged(opt)
-- General
 if opt == "show_mapicons" then
  maps:UpdateIcons("worldmap")
 elseif opt == "show_minimapicons" then
  maps:UpdateIcons("minimap")
-- Map Icons UI
 elseif opt == "worldmap_icon_interact" then
  maps:UpdateInteractable("worldmap")
 elseif opt == "minimap_icon_interact" then
  maps:UpdateInteractable("minimap")
 elseif opt == "worldmap_icon_size" then
  maps:UpdateIconsSize("worldmap")
 elseif opt == "minimap_icon_size" then
  maps:UpdateIconsSize("minimap")
 end
end

local function createOptionsFrame(p)

 local of = CreateFrame("Frame", "DGCOOptionsFrame", p)
 of:SetAllPoints()

 local chkcounter = 0

 local function CreateCheckbox(opt, text)

  local f = constructors:Checkbox(of, p:GetWidth() - 40, text)

  table.insert(of.checkboxes, f)
  f:SetPoint("TOPLEFT", of, "TOPLEFT", 10, frameoffsetY + -20 * chkcounter)

  f:SetScript("OnClick", function(self)
    if opt == "minimap_icon" then
      DailyGlobalCheck_Options["minimap_icon"].hide = not self:GetChecked()
 	 addonTable.ldbicon:Refresh("DailyGlobalCheck_broker", DailyGlobalCheck_broker, DailyGlobalCheck_Options["minimap_icon"])
    else
     DailyGlobalCheck_Options[opt] = self:GetChecked()
	 options:OnOptionChanged(opt)
	 DailyGlobalCheck:Refresh()
    end
  end)

  if opt == "minimap_icon" then
   f:SetChecked(DailyGlobalCheck_Options["minimap_icon"] and not DailyGlobalCheck_Options["minimap_icon"].hide or false)
  else
   f:SetChecked(DailyGlobalCheck_Options[opt])
  end
  chkcounter = chkcounter + 1
  return f
 end

 of.checkboxes = {}
 CreateCheckbox("minimap_icon",L["opt_minimap"])
 chkcounter = chkcounter + 1
 CreateCheckbox("show_questtype", L["opt_questtype"])
 CreateCheckbox("show_prefix", L["opt_prefix"])
 CreateCheckbox("show_zone", L["opt_zonename"])
 CreateCheckbox("show_suffix", L["opt_suffix"])
 CreateCheckbox("show_coordinates", L["opt_coords"])
 CreateCheckbox("hide_completed", L["opt_hidecompleted"])
 chkcounter = chkcounter + 2
 CreateCheckbox("show_questlogbtn", L["opt_showquestbtn"])
 chkcounter = chkcounter + 2
 CreateCheckbox("show_mapicons", L["opt_map"])
 CreateCheckbox("show_minimapicons", L["opt_minimap_icons"])
 return of
end

------ Plugin Options ------
local opt_plugin_title, lastselected

-- widgets handling
local widgets_pool = {}
local function getwidget(t)
 local found
 for k,v in pairs(widgets_pool) do
  if v.t == t then
   found = k
   break
  end
 end

 if found then
  local f = widgets_pool[found]
  table.remove(widgets_pool, found)
  table.insert(DGCOPluginOptionsFrame.widgets, f)
  return f
 elseif t == "Checkbox" or t == "Radiobutton" then
  local f = constructors:Checkbox(DGCOPluginOptionsFrame, DGCOptionsFrame:GetWidth() - 40, "")
  table.insert(DGCOPluginOptionsFrame.widgets, f)
  f.t = t
  return f
 end
end
--

local function updatepluginframe()
 local list = DailyGlobalCheck:GetSelectedList()
 local of = DGCOPluginOptionsFrame

 if not of or list == lastselected then return end
 
  for _,v in pairs(of.widgets) do
   v:Hide()
   table.insert(widgets_pool, v)
  end
  wipe(of.widgets)

 if not list or not list["Options"] then
  --opt_plugin_title:SetText(L["plugin_opt_title"].."\n\n"..L["plugin_opt_none"])
  opt_plugin_title:SetText(L["plugin_opt_title"])
 else
  opt_plugin_title:SetText(L["plugin_opt_title"])

  -- widgets creation
  if not DailyGlobalCheck_PluginData[list["Title"]] then DailyGlobalCheck_PluginData[list["Title"]] = {} end
  local svar = DailyGlobalCheck_PluginData[list["Title"]]
  if not svar.options then svar.options = {} end

  for i,v in pairs(list["Options"]) do
   local t, text, refresh, flags = unpack(v)
   local f = getwidget(t)
   f.tag = i
   f:SetPoint("TOPLEFT", of, "TOPLEFT", 10, frameoffsetY + -20 * i)
   
   -- Checkbox
   if t == "Checkbox" then
    f.title:SetText(text)
    f:SetChecked(svar.options[i])
	
    f:SetScript("OnClick", function(self)

     if not svar.options then svar.options = {} end
	 svar.options[self.tag] = f:GetChecked()
	 
	 if list.OnOptionChanged then
	  list.OnOptionChanged(self.tag)
	 end
	 
	 if refresh then
	  DailyGlobalCheck:Refresh()
     end
    end)
   -- Radiobutton
   elseif t == "Radiobutton" then
    local category = flags[1] or 0
	
    f.title:SetText(text)
    f:SetChecked(svar.options[i])
	f.category = category
	
    f:SetScript("OnClick", function(self)

     if not svar.options then svar.options = {} end
	 
	 for wi,ww in pairs(of.widgets) do
	  if ww.category == category and ww ~= f then
	   ww:SetChecked(false)
	   svar.options[wi] = false
	  end
	 end

	 svar.options[i] = true
	 f:SetChecked(true)
	
	 if refresh then
	  DailyGlobalCheck:Refresh()
     end
    end)
   end
   f:Show()
  end
 end
 
 -- map icons checkbox
 if list then
  local svar = lists:GetSvar(list)
  if svar.show_mapicons_worldmap == nil then 
   lists:setShowMapIcons(list, "worldmap", true)
  end
  if svar.show_mapicons_minimap == nil then 
   lists:setShowMapIcons(list, "minimap", true)
  end
  of.showworldmapicons:SetChecked(svar.show_mapicons_worldmap)
  of.showminimapicons:SetChecked(svar.show_mapicons_minimap)
 end
 
 lastselected = list
end

local function createPluginFrame(p)

 local of = CreateFrame("Frame", "DGCOPluginOptionsFrame", p)
 of:SetAllPoints()
 
 local t = of:CreateFontString(nil, "OVERLAY", "GameTooltipText")
 t:SetPoint("TOPLEFT", of, "TOPLEFT", 10, -50)
 opt_plugin_title = t
 of.widgets = {}
 
 t:SetWordWrap(true)
 t:SetWidth(p:GetWidth() - 20)
 
 local f = constructors:Checkbox(of, p:GetWidth() - 40, L["plugin_mapicons"])
 f:SetPoint("BOTTOMLEFT", of, "BOTTOMLEFT", 10, 50)
 f:SetScript("OnClick", function(self)
     lists:setShowMapIcons(DailyGlobalCheck:GetSelectedList(), "worldmap", self:GetChecked())
    end)
 of.showworldmapicons = f
 
 f = constructors:Checkbox(of, p:GetWidth() - 40, L["plugin_mapicons_minimap"])
 f:SetPoint("BOTTOMLEFT", of, "BOTTOMLEFT", 10, 10)
 f:SetScript("OnClick", function(self)
     lists:setShowMapIcons(DailyGlobalCheck:GetSelectedList(), "minimap", self:GetChecked())
    end)
 of.showminimapicons = f
  
 updatepluginframe()
 return of
end

function options:OnDraw()
 updatepluginframe()
end

local frames_backdrop  = { bgFile = "Interface\\Buttons\\GreyscaleRamp64", 
                           edgeFile = "Interface\\Buttons\\WHITE8X8", 
						   edgeSize = 1
						   }

----------------
function options:CreateOptionsFrame(parent)
 local f = CreateFrame("Frame","DGCOptionsFrame",parent)
 f:SetFrameStrata("HIGH")
 f:SetBackdrop(frames_backdrop)
 f:SetBackdropColor(0.07,0.07,0.07,0.9)
 f:SetBackdropBorderColor(0.1,0.1,0.1,0.9)
 f:SetPoint("LEFT", parent, "RIGHT")
 f:SetHeight(390)
 f:SetWidth(200)

 local function onmouseup(self)
  for _,v in pairs(f.buttons) do
   v:SetBackdropColor(0,0,0,0.9)
   v.panel:Hide()
  end
  self:SetBackdropColor(0,1,0,0.9)
  self.panel:Show()
 end
 
 f.buttons = {}

 -- Options Frame
 f.Options = createOptionsFrame(f)
 
 local btn = constructors:largebuttontemplate(f, GAMEOPTIONS_MENU, "")
 btn:SetBackdropColor(0,1,0,0.9)
 btn:SetPoint("TOPLEFT")
 btn.panel = f.Options
 btn:SetScript("OnMouseUp", onmouseup)
 f.buttons.btnopt = btn

-- Plugin Options Frame
 f.Plugin = createPluginFrame(f)
 f.Plugin:Hide()
 
 btn = constructors:largebuttontemplate(f, "Plugin", "")
 btn:SetBackdropColor(0,0,0,0.9)
 btn:SetPoint("LEFT", f.buttons.btnopt, "RIGHT")
 btn.panel = f.Plugin
 btn:SetScript("OnMouseUp", onmouseup)
 f.buttons.btnplugin = btn

 f:Hide()
 return f
end

-- Blizzard Interface Panel --
local widgets = {}

-- Daily Global Check (parent)
local function generateMainOptions(p)
--
end

-- UI
local function generateUIOptions(self)
 if not self.initialized then
  self.initialized = true
 end

end

-- Map Icons
local function generateMapOptions(self)
 if not self.initialized then
  
  local function createMapGroup(t)
   local grp = constructors:Groupbox(self, nil, 200, 100, string.upper(t))

   local O = DailyGlobalCheck_Options.UI

   -- interactable worldmap icons
   local f = constructors:Checkbox(grp, 140, MAKE_INTERACTABLE)
   f.title:SetJustifyH("LEFT")
   f:SetPoint("TOPLEFT", grp, "TOPLEFT", 10, -10)
   f:SetChecked(O[t.."_icon_interact"])
   widgets[t.."_icon_interact"] = f

   -- worldmap icon size
   f = constructors:Slider(grp, "DGC"..t.."IconSizeSlider", "", C.mapicons_size_min, C.mapicons_size_max, function(self, value)
                                                                      self.preview_icon:SetSize(value, value)
																	  self.lbl:SetText(L["size"].." "..AddColor(math.floor(value), "LIGHT_GREEN"))
                                                                     end)

   f.preview_icon = self:CreateTexture()
   f.preview_icon:SetPoint("LEFT", f, "RIGHT", 20, 0)
   f.preview_icon:SetTexture(dgcicon)
   f:SetPoint("TOPLEFT", grp, "TOPLEFT", 10, -50)
   f:SetValue(O[t.."_icon_size"])
   widgets[t.."_icon_size"] = f
   return grp
  end

  -- world map
  local grp = createMapGroup("worldmap")
  grp:SetPoint("TOPLEFT", self, "TOPLEFT", 10, -60)
  grp:SetPoint("TOPRIGHT", self, "TOP", -10, -60)
  -- minimap
  grp = createMapGroup("minimap")
  grp:SetPoint("TOPLEFT", self, "TOP", 10, -60)
  grp:SetPoint("TOPRIGHT", self, "TOPRIGHT", -10, -60)
  
  self.initialized = true
 end
end

local function OkButtonClick()
 local O = DailyGlobalCheck_Options.UI
 for k,v in pairs(widgets) do
  if v.GetChecked then
   O[k] = v:GetChecked()
  elseif v.GetValue then
   O[k] = v:GetValue()
  end
  options:OnOptionChanged(k)
 end
end

local function updateWidgets()
 local O = DailyGlobalCheck_Options.UI
 for k,v in pairs(widgets) do
  if v.SetChecked then
   v:SetChecked(O[k])
  elseif v.SetValue then
   v:SetValue(O[k])
  end
 end
end

local function DefaultButtonClick()
 local O = DailyGlobalCheck_Options.UI
 for k,v in pairs(C.defaults.UI) do
  O[k] = v
  options:OnOptionChanged(k)
 end
end

-- child panel creation
local counter = 0
local function createChildPanel(name, func)
 counter = counter + 1
 local f = CreateFrame("Frame", "DGC_BlizzOptionsPanel"..counter, UIParent)
 f:Hide()
 f.title = f:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge")
 f.title:SetJustifyH("LEFT")
 f.title:SetJustifyV("TOP")
 f.title:SetPoint("TOPLEFT", f, "TOPLEFT", 16, -16)
 f.title:SetPoint("BOTTOMRIGHT", f, "TOPRIGHT", 16, -64)
 f.title:SetText("Daily Global Check - "..name)
 f.name   = name
 f.parent = "Daily Global Check"
 f:SetScript("OnShow", func)
 f.default = DefaultButtonClick
 f.refresh = updateWidgets
 InterfaceOptions_AddCategory(f)
end
-----------------------

DailyGlobalCheck.BlizzOptionsPanel = CreateFrame("Frame", "DGC_BlizzOptionsPanel", UIParent)
local f = DailyGlobalCheck.BlizzOptionsPanel
f:Hide()
f.title = f:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge")
f.title:SetJustifyH("LEFT")
f.title:SetJustifyV("TOP")
f.title:SetPoint("TOPLEFT", f, "TOPLEFT", 16, -16)
f.title:SetPoint("BOTTOMRIGHT", f, "TOPRIGHT", 16, -64)
f.title:SetText("Daily Global Check")
f.name = "Daily Global Check"
f.generate = generateMainOptions
f:SetScript("OnShow", f.generate)
f.okay    = OkButtonClick
f.cancel  = updateWidgets
InterfaceOptions_AddCategory(f)

--createChildPanel("UI", generateUIOptions)
createChildPanel("Map Icons", generateMapOptions)